// Problem   : Crossword
// Generator : Igor V. Burago

#include <iostream>
#include <iterator>
#include <string>
#include <vector>

using namespace std;

const size_t MinWordLength = 3;

int Width;
int Height;
char **Field;
int **CellCount;
vector<string> Words;
bool Incorrect;

inline bool RightIndecies(const int Col, const int Row)
{
  return 0 <= Col && Col < Width && 0 <= Row && Row < Height;
}

void CheckWord(
  const bool Horizontal,
  const int StartCol,
  const int StartRow)
{
  static int DeltaX[2] = { 0, 1 };
  static int DeltaY[2] = { 1, 0 };

  size_t Length = 0;
  int Row = StartRow;
  int Col = StartCol;
  while (RightIndecies(Col, Row) && Field[Row][Col] != '.')
  {
    ++Length;
    Row += DeltaY[Horizontal];
    Col += DeltaX[Horizontal];
  }

  if (Length >= MinWordLength)
  {
    Row = StartRow;
    Col = StartCol;
    while (RightIndecies(Col, Row) && Field[Row][Col] != '.')
    {
      ++CellCount[Row][Col];
      Row += DeltaY[Horizontal];
      Col += DeltaX[Horizontal];
    }
  }
}

void FillWord(
  const bool Horizontal,
  const int StartCol,
  const int StartRow)
{
  static int DeltaX[2] = { 0, 1 };
  static int DeltaY[2] = { 1, 0 };

  string Word;
  int Row = StartRow;
  int Col = StartCol;
  while (RightIndecies(Col, Row) && Field[Row][Col] != '.')
  {
    if (Incorrect && CellCount[Row][Col] > 1)
      Word += 'a' + rand() % 26;
    else
      Word += Field[Row][Col];
    Row += DeltaY[Horizontal];
    Col += DeltaX[Horizontal];
  }
  if (Word.length() >= MinWordLength)
    Words.push_back(Word);
}

int main(int argc, char *argv[])
{
  unsigned RandStart;
  if (
    argc < 5 || 
    sscanf(argv[1], "%u", &RandStart) != 1 ||
    sscanf(argv[2], "%d", &Width) != 1 ||
    sscanf(argv[3], "%d", &Height) != 1)
    return 1;

  #ifdef CROSSWORD_NUMBERED
  int KeywordLength;
  if (argc < 6 || sscanf(argv[5], "%d", &KeywordLength) != 1)
    KeywordLength = 1 + rand() % (MaxKeywordLength - 1);
  #endif

  srand(RandStart);

  int Row;
  int Col;

  Field = new char *[Height];
  CellCount = new int *[Height];
  for (Row = 0; Row < Height; ++Row)
  {
    Field[Row] = new char[Width];
    CellCount[Row] = new int[Width];
  }


  int SpaceCount = 0;
  bool NotSpace;

  for (Row = 0; Row < Height; ++Row)
    for (Col = 0; Col < Width; ++Col)
    {
      NotSpace = (Row % 2 != 0 || Col % 2 != 0);
      Field[Row][Col] = " #"[NotSpace];
      SpaceCount += !NotSpace;
      CellCount[Row][Col] = 0;
    }

  const int DeltaX[4] = { 0, 1, 0, -1 };
  const int DeltaY[4] = { -1, 0, 1, 0 };

  int Dir;

  Field[0][0] = '.';
  --SpaceCount;
  while (SpaceCount > 0)
    for (Row = 0; Row < Height; ++Row)
      for (Col = 0; Col < Width; ++Col)
      {
        Dir = rand() % 4;
        if (
          Field[Row][Col] == '.' &&
          RightIndecies(Col + 2*DeltaX[Dir], Row + 2*DeltaY[Dir]) &&
          Field[Row + 2*DeltaY[Dir]][Col + 2*DeltaX[Dir]] == ' ')
        {
          SpaceCount -= (Field[Row + DeltaY[Dir]][Col + DeltaX[Dir]] == ' ');
          Field[Row + 2*DeltaY[Dir]][Col + 2*DeltaX[Dir]] = '.';
          Field[Row + DeltaY[Dir]][Col + DeltaX[Dir]] = ".#"[rand() % 2];
          --SpaceCount;
        }
      }

  if (Width % 2 == 0)
  {
    for (Row = 0; Row < Height; ++Row)
      if (Field[Row][Width - 2] == '.' && rand() % 4 == 0)
        Field[Row][Width - 1] = '.';
  }
  if (Height % 2 == 0)
  {
    for (Col = 0; Col < Width; ++Col)
      if (Field[Height - 2][Col] == '.' && rand() % 4 == 0)
        Field[Height - 1][Col] = '.';
  }

  for (Row = 0; Row < Height; ++Row)
    for (Col = 0; Col < Width; ++Col)
    {
      if (
        Field[Row][Col] != '.' && 
        (Col == 0 || (Col > 0 && Field[Row][Col - 1] == '.')))
        CheckWord(true, Col, Row);
      if (
        Field[Row][Col] != '.' && 
        (Row == 0 || (Row > 0 && Field[Row - 1][Col] == '.')))
        CheckWord(false, Col, Row);
    }

  for (Row = 0; Row < Height; ++Row)
    for (Col = 0; Col < Width; ++Col)
      if (CellCount[Row][Col] == 0)
        Field[Row][Col] = '.';


  if (strcmp(argv[4], "random") == 0)
    Incorrect = rand() % 2;
  else
    Incorrect = (strcmp(argv[4], "incorrect") == 0);

  #ifdef CROSSWORD_NUMBERED
  int LettersCount = 0;
  #endif

  for (Row = 0; Row < Height; ++Row)
    for (Col = 0; Col < Width; ++Col)
      if (Field[Row][Col] == '#')
      {
        Field[Row][Col] = 'a' + rand() % 26;
        #ifdef CROSSWORD_NUMBERED
        ++LettersCount;
        #endif
      }

  for (Row = 0; Row < Height; ++Row)
    for (Col = 0; Col < Width; ++Col)
      if (
        Field[Row][Col] != '.' && 
        (Col == 0 || (Col > 0 && Field[Row][Col - 1] == '.')))
        FillWord(true, Col, Row);

  for (Row = 0; Row < Height; ++Row)
    for (Col = 0; Col < Width; ++Col)
      if (
        Field[Row][Col] != '.' && 
        (Row == 0 || (Row > 0 && Field[Row - 1][Col] == '.')))
        FillWord(false, Col, Row);


  cout << Width << ' ' << Height << endl;

  #ifdef CROSSWORD_NUMBERED
  int KeysCount = KeywordLength;
  for (Row = 0; Row < Height; ++Row)
  {
    for (Col = 0; Col < Width; ++Col)
      if (Field[Row][Col] != '.')
      {
        if ((long long)LettersCount * rand() < (long long)(KeysCount * RAND_MAX))
        {
          putchar('1' + KeywordLength - KeysCount);
          --KeysCount;
        }
        else
          putchar('#');
        --LettersCount;
      }
      else
        putchar('.');
    putchar('\n');
  }
  #else
  for (Row = 0; Row < Height; ++Row)
  {
    for (Col = 0; Col < Width; ++Col)
      putchar(".#"[Field[Row][Col] != '.']);
    putchar('\n');
  }
  #endif

  cout << Words.size() << endl;
  copy(Words.begin(), Words.end(), ostream_iterator<string>(cout, "\n"));


  for (Row = 0; Row < Height; ++Row)
  {
    delete [] Field[Row];
    delete [] CellCount[Row];
  }
  delete [] Field;
  delete [] CellCount;

  return 0;
}
