// Problem  : Crossword
// Solution : Igor V. Burago

#include <fstream>
#include <iomanip>
#include <iterator>
#include <vector>
#include <string>
#include <algorithm>
#include <cassert>

using namespace std;

const size_t MinWordLength = 3;
const size_t MaxKeywordLength = 9;

ifstream cin("input.txt");
ofstream cout("output.txt");

int Width;
int Height;
vector<string> Field;
vector<string> Words;
char Keyword[MaxKeywordLength + 1] = { '\0' };

/* NOTE:
 *   It's not necessary to read all words into memory to solve the problem.
 *   I did it only to be sure that any solution will have enough memory to work.
 */

void Terminate(const string &Word)
{
  #ifdef CROSSWORD_NUMBERED
  cout << "INCORRECT" << endl;
  #else
  cout << Word << endl;
  #endif
  exit(0);
}

inline bool RightIndecies(const int Col, const int Row)
{
  return 0 <= Col && Col < Width && 0 <= Row && Row < Height;
}

bool FillWord(
  const bool Horizontal,
  const int StartCol,
  const int StartRow,
  const string &Word)
{
  static const int DeltaX[2] = { 0, 1 };
  static const int DeltaY[2] = { 1, 0 };

  size_t Length = 0;
  int Row = StartRow;
  int Col = StartCol;
  while (RightIndecies(Col, Row) && Field[Row][Col] != '.')
  {
    ++Length;
    Row += DeltaY[Horizontal];
    Col += DeltaX[Horizontal];
  }
  if (Length < MinWordLength)
    return false;

  if (Length != Word.length())
    Terminate(Word);

  Length = 0;
  Row = StartRow;
  Col = StartCol;
  while (RightIndecies(Col, Row) && Field[Row][Col] != '.')
  {
    if (isalpha(Field[Row][Col]) && Field[Row][Col] != Word[Length])
      Terminate(Word);

    assert('a' <= Word[Length] && Word[Length] <= 'z');
    if (isdigit(Field[Row][Col]))
      Keyword[Field[Row][Col] - '1'] = Word[Length];
    Field[Row][Col] = Word[Length];
    ++Length;
    Row += DeltaY[Horizontal];
    Col += DeltaX[Horizontal];
  }

  assert(Length <= 250);
  
  return true;
}

int main()
{
  cin >> Width >> Height >> ws;
  assert(3 <= Width && Width <= 500);
  assert(3 <= Height && Height <= 500);

  Field.resize(Height);
  for (int Row = 0; Row < Height; ++Row)
  {
    getline(cin, Field[Row]);
    assert(Field[Row].length() == (size_t)Width);
  }

  int Count;
  cin >> Count >> ws;
  Words.reserve(Count);
  copy(istream_iterator<string>(cin), istream_iterator<string>(), back_inserter(Words));
  assert(Words.size() == (size_t)Count);
  assert(0 < Count && Count <= 30000);

  int Row;
  int Col;
  Count = 0;

  for (Row = 0; Row < Height; ++Row)
    for (Col = 0; Col < Width; ++Col)
      if (
        Field[Row][Col] != '.' &&
        (Col == 0 || (Col > 0 && Field[Row][Col - 1] == '.')) &&
        (size_t)Count < Words.size() && FillWord(true, Col, Row, Words[Count]))
          ++Count;

  for (Row = 0; Row < Height; ++Row)
    for (Col = 0; Col < Width; ++Col)
      if (
        Field[Row][Col] != '.' &&
        (Row == 0 || (Row > 0 && Field[Row - 1][Col] == '.')) &&
        (size_t)Count < Words.size() && FillWord(false, Col, Row, Words[Count]))
          ++Count;

  #ifdef _DEBUG
  for (Row = 0; Row < Height; ++Row)
  {
    for (Col = 0; Col < Width; ++Col)
      cout << Field[Row][Col];
    cout << endl;
  }
  #endif

  #ifdef CROSSWORD_NUMBERED
  assert(*Keyword != '\0');
  cout << Keyword << endl;
  #else
  assert(*Keyword == '\0');
  cout << "CORRECT" << endl;
  #endif

  return 0;
}
